#if defined(SIMDE_TESTS_RUN_TESTS_H)
  #error File already included.
#endif
#define SIMDE_TESTS_RUN_TESTS_H

#include "test.h"
#include "arm/run-tests.h"
#include "x86/run-tests.h"
#include "wasm/run-tests.h"
#include "mips/run-tests.h"
